package com.faraphel.tasks_valider

import android.content.Context
import android.net.wifi.p2p.*
import android.net.wifi.p2p.WifiP2pManager.ActionListener
import android.net.wifi.p2p.WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION
import android.net.wifi.p2p.WifiP2pManager.WIFI_P2P_STATE_ENABLED
import android.os.Build
import android.os.Bundle
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.annotation.RequiresApi
import com.faraphel.tasks_valider.database.Database
import java.net.InetAddress
import java.net.InetSocketAddress
import java.net.ServerSocket
import java.net.Socket


class MainActivity : ComponentActivity() {
    private var p2pManager: WifiP2pManager? = null
    private var p2pChannel: WifiP2pManager.Channel? = null

    companion object {
        private lateinit var database: Database
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // ----- CONNECTION

        /*
        val intentFilter = IntentFilter()

        // Indicates a change in the Wi-Fi Direct status.
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION)
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION)
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION)
        intentFilter.addAction(WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION)

        this.registerReceiver(MyBroadcastReceiver(this), intentFilter)
        */

        // get the WiFi-Direct manager
        this.p2pManager = this.getSystemService(Context.WIFI_P2P_SERVICE) as WifiP2pManager?
        if (this.p2pManager == null) {
            Log.e("wifi-p2p", "cannot access the WiFi-Direct manager")
            return
        }

        // get the Wifi-Direct channel
        this.p2pChannel = this.p2pManager!!.initialize(this, this.mainLooper, null)

        // test the current channel
        this.callWithP2pPeers { devices ->
            // DEBUG
            Log.d("wifi-p2p", "Peers found : ${devices.size}")
            devices.forEach { device ->
                Log.d(
                    "wifi-p2p",
                    "peer found : [${device.deviceAddress}] ${device.deviceName}"
                )
            }
        }

        this.callWithP2pGroup { group ->
            Log.d("wifi-p2p", "Is group owner : ${group.isGroupOwner}")
        }

        /*
        // NOTE(Faraphel): it seem that this feature is deprecated and should be done manually.
        // get the list of peers in the group
        manager.discoverPeers(channel, object : ActionListener {
            override fun onSuccess() {
                Log.d("wifi-p2p", "Peers discovery started")
            }

            override fun onFailure(reason: Int) {
                Log.e("wifi-p2p", "Error when starting peers discovery : $reason")
                // NOTE: might fail if the "Location" permission is not granted
            }
        })
        */
    }

    private fun callWithP2pPeers(callback: (Collection<WifiP2pDevice>) -> Unit) {
        this.p2pManager?.requestPeers(this.p2pChannel) { devices ->
            // NOTE(Faraphel): might be empty if the "Location" permission is not granted
            if (devices.deviceList.isEmpty())
                Log.w("wifi-p2p", "No WiFi-Direct devices found. Have you enabled, granted required permissions and created the group ?")

            callback(devices.deviceList)
        }
    }

    private fun callWithP2pGroup(callback: (WifiP2pGroup) -> Unit) {
        this.p2pManager?.requestGroupInfo(this.p2pChannel) { group ->
            callback(group)
        }
    }
}
